<?php
/*
Plugin Name: Ballast Security Hashing
Plugin URI:  http://wordpress.org/extend/plugins/ballast-security-securing-hashing/
Description: Replaces the login hash of the WordPress with 2048 iterations of a modified PBKDF2 using SHA-256 and 16 bytes of salt the SHA1'd to be shortened
Author: <a href="https://www.twitter.com/bwallHatesTwits/" target="_blank">@bwallHatesTwits</a>
Version: 0.3.beta
License: GPLv2
*/

class BallastPHPHash
{
	function BSPBKDF2($plain, $salt, $iterations = 2048, $algo = 'sha256' ) 
	{
		$derivedkey = $b = hash_hmac($algo, $salt, $plain, true);
		for ( $i = 0; $i < $iterations; $i++ )
		{
			$derivedkey = hash_hmac($algo, $b, $plain, true);
		}
		return sha1($derivedkey, true);
	}
	
	//Hash Format - $BPBK$Iterations$Salt$Hash
	public function HashUpToDate($hash)
	{
		$type = get_option("BallastSecurityHashType");
		if($type === false)
		{
			//option is not defined
			add_option("BallastSecurityHashType", '$BPBK$2048$', "", "yes");
			$type = get_option("BallastSecurityHashType");
		}
		//Default WordPress is '$P$'
		return (substr($hash, 0, strlen($type)) == $type);
	}
	
	function StartsWith($hash, $type)
	{
		return (substr($hash, 0, strlen($type)) == $type);
	}
	
	public function CheckPassword($password, $hash)
	{
		if($this->StartsWith($hash, '$BPBK$2048$'))
		{
			$saltAndhash = substr($hash, 11);
			$salt = strstr($saltAndhash, '$', true);
			$hash = substr(strstr($saltAndhash, '$'), 1);
			$realHash = base64_encode($this->BSPBKDF2($password, base64_decode($salt)));
			return ($hash == $realHash);
		}
		else if($this->StartsWith($hash, '$BPBK$10k$'))
		{
			$saltAndhash = substr($hash, 10);
			$salt = strstr($saltAndhash, '$', true);
			$hash = substr(strstr($saltAndhash, '$'), 1);
			$realHash = base64_encode($this->BSPBKDF2($password, base64_decode($salt), 10000));
			return ($hash == $realHash);
		}
		else if($this->StartsWith($hash, '$BPBK$100k$'))
		{
			$saltAndhash = substr($hash, 11);
			$salt = strstr($saltAndhash, '$', true);
			$hash = substr(strstr($saltAndhash, '$'), 1);
			$realHash = base64_encode($this->BSPBKDF2($password, base64_decode($salt), 100000));
			return ($hash == $realHash);
		}
		else if($this->StartsWith($hash, '$P$'))
		{
			require_once(ABSPATH.'wp-includes/class-phpass.php');
			$ph = new PasswordHash(8, TRUE);
			return $ph->CheckPassword($password, $hash);
		}
	}
	
	public function HashPassword($password)
	{
		$type = get_option("BallastSecurityHashType");
		if($type === false)
		{
			//option is not defined
			add_option("BallastSecurityHashType", '$BPBK$2048$', "", "yes");
			$type = get_option("BallastSecurityHashType");
		}
		if($type === '$BPBK$2048$')
		{
			$hash = '$BPBK$2048$';
			$salt = "";
			for($i = 0; $i < 16; $i++)
			{
				$salt .= chr(rand(0, 256));
			}
			$hash .= base64_encode($salt).'$'.base64_encode($this->BSPBKDF2($password, $salt));
			return $hash;
		}
		else if($type === '$BPBK$10k$')
		{
			$hash = '$BPBK$10k$';
			$salt = "";
			for($i = 0; $i < 16; $i++)
			{
				$salt .= chr(rand(0, 256));
			}
			$hash .= base64_encode($salt).'$'.base64_encode($this->BSPBKDF2($password, $salt, 10000));
			return $hash;
		}
		else if($type === '$BPBK$100k$')
		{
			$hash = '$BPBK$100k$';
			$salt = "";
			for($i = 0; $i < 16; $i++)
			{
				$salt .= chr(rand(0, 256));
			}
			$hash .= base64_encode($salt).'$'.base64_encode($this->BSPBKDF2($password, $salt, 100000));
			return $hash;
		}
		else if($type === '$P$')
		{
			require_once(ABSPATH.'wp-includes/class-phpass.php');
			$ph = new PasswordHash(8, TRUE);
			return $ph->HashPassword($password);
		}
	}
}
if (!function_exists('wp_hash_password'))
{
	function wp_hash_password($password) 
	{
		global $wp_hasher;
		if ( empty($wp_hasher) ) 
		{
			$wp_hasher = new BallastPHPHash();
		}
		return $wp_hasher->HashPassword($password);
	}
}

if (!function_exists('wp_check_password'))
{
	function wp_check_password($password, $hash, $user_id = '') 
	{
		global $wp_hasher;
		$wp_hasher = new BallastPHPHash();	
		if ( strlen($hash) <= 32 ) 
		{
			$check = ( $hash == md5($password) );
			if ( $check && $user_id ) 
			{
				wp_set_password($password, $user_id);
				$hash = wp_hash_password($password);
			}
			return apply_filters('check_password', $check, $password, $hash, $user_id);
		}
		
		if(!$wp_hasher->HashUpToDate($hash))
		{
			$check = $wp_hasher->CheckPassword($password, $hash);
			if($check && $user_id)
			{
				wp_set_password($password, $user_id);
				$hash = wp_hash_password($password);
			}
			return apply_filters('check_password', $check, $password, $hash, $user_id);
		}
		$check = $wp_hasher->CheckPassword($password, $hash);	
		return apply_filters('check_password', $check, $password, $hash, $user_id); 
	}
}

function ballastsec_hash_menu() 
{
	add_menu_page('Ballast Security Secure Hasher', 'Secure Hasher Configuration', 'add_users','bssh_config', 'display_bssh_config' );
}

add_action('admin_menu', 'ballastsec_hash_menu');

function display_bssh_config() 
{	
	if(isset($_POST['hashtype']))
	{
		$type = get_option("BallastSecurityHashType");
		if($_POST['hashtype'] == "1")
		{
			if($type === false)
			{
				add_option("BallastSecurityHashType", '$BPBK$2048$', "", "yes");
			}
			else
			{
				update_option("BallastSecurityHashType", '$BPBK$2048$');
			}
		}
		else if($_POST['hashtype'] == "2")
		{
			if($type === false)
			{
				add_option("BallastSecurityHashType", '$P$', "", "yes");
			}
			else
			{
				update_option("BallastSecurityHashType", '$P$');
			}
		}
		else if($_POST['hashtype'] == "3")
		{
			if($type === false)
			{
				add_option("BallastSecurityHashType", '$BPBK$10k$', "", "yes");
			}
			else
			{
				update_option("BallastSecurityHashType", '$BPBK$10k$');
			}
		}
		else if($_POST['hashtype'] == "4")
		{
			if($type === false)
			{
				add_option("BallastSecurityHashType", '$BPBK$100k$', "", "yes");
			}
			else
			{
				update_option("BallastSecurityHashType", '$BPBK$100k$');
			}
		}
	}
	$type = get_option("BallastSecurityHashType");
	if($type === false)
	{
		//option is not defined
		add_option("BallastSecurityHashType", '$BPBK$2048$', "", "yes");
		$type = get_option("BallastSecurityHashType");
	}
	$bpk = "";
	$bpk10k = "";
	$bpk100k = "";
	$wp = "";
	if($type == '$BPBK$2048$')
	{
		$bpk = "checked=\"true\"";
	}
	else if($type == '$BPBK$10k$')
	{
		$bpk10k = "checked=\"true\"";
	}
	else if($type == '$BPBK$100k$')
	{
		$bpk100k = "checked=\"true\"";
	}
	else if($type == '$P$')
	{
		$wp = "checked=\"true\"";
	}
	
	echo "<h2>Pick your hash type</h2><br />";
	echo "<form method='POST'>";
	echo "<input type=\"radio\" name=\"hashtype\" value=\"1\" ".$bpk."/> Use Ballast Security's modified PBKDF2 with 2048 iterations<br />";
	echo "<input type=\"radio\" name=\"hashtype\" value=\"3\" ".$bpk10k."/> Use Ballast Security's modified PBKDF2 with 10000 iterations<br />";
	echo "<input type=\"radio\" name=\"hashtype\" value=\"4\" ".$bpk100k."/> Use Ballast Security's modified PBKDF2 with 100000 iterations<br />";
	echo "<input type=\"radio\" name=\"hashtype\" value=\"2\" ".$wp."/> Use default that comes with WordPress<br />";
	echo "<input type=\"submit\" value=\"Save Hash Type\" /><br /></form>";
	echo "<br />Note: If you want to deactive this plugin, you must change your settings over to use the default, and make sure all your users login in again so their hashes can be converted back.<br />";
}
?>
